/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};
var catchedState = '',
  revIdText = `does not exist`,
  firmwareVersion = 'unknown';

var appResetFunction = function () {
  let appDataComponent = document.querySelector('#ti_mdbu_custom_appdata');
  appDataComponent._setDeviceConnectionState({
    deviceId: '',
    revId: undefined,
    isConnected: false
  });

  if (gc.connectionManager.status == "connected") {
    gc.connectionManager.disconnect();
  }
  templateObj.$.ti_widget_menubar.productName = originalProductName;
  document.querySelector('#ti_widget_custom_multidevice_appmgr').resetApp();
  resetHighLevelPage(true, appDataComponent);
  updateFileMenuOptionsVisibility(true);
  setTimeout(function () {
    gc.connectionManager.connect();
  }, 1000);
};

var updateFileMenuOptionsVisibility = function (isOnLandingPage) {
  let programDeviceMenu = document.querySelector('#FileProgram');
  let loadRegOption = document.querySelector('#load_registers');
  let saveRegOption = document.querySelector('#save_registers');
  let loadGUIRegOption = document.querySelector('#load_gui_register');
  let menuAction = programDeviceMenu.querySelector('#menuAction');

  if (isOnLandingPage) {
    if (programDeviceMenu && menuAction) {
      menuAction.style.display = "";
    }
    if (loadRegOption) {
      loadRegOption.style.display = "none";
    }
    if (saveRegOption) {
      saveRegOption.style.display = "none";
    }
    if (loadGUIRegOption) {
      loadGUIRegOption.style.display = "none";
    }
  } else {
    if (programDeviceMenu && menuAction) {
      menuAction.style.display = "none";
    }
    if (loadRegOption) {
      loadRegOption.style.display = "";
    }
    if (saveRegOption) {
      saveRegOption.style.display = "";
    }
    if (loadGUIRegOption) {
      loadGUIRegOption.style.display = "";
    }
  }
}

var resetHighLevelPage = function (isExit, appDataComponent) {
  resetHVACPage(isExit, appDataComponent);
  resetSideMirrorPage(isExit, appDataComponent);
}

var resetHVACPage = function (isExit, appDataComponent) {
  var mainPageElement = document.querySelector('#ti_custom_motor_control_main');
  if (mainPageElement) {
    mainPageElement.index = -1;
  }
  var configPageElements = document.querySelectorAll('#ti_custom_motor_control_config');
  for (let i = 0; i < configPageElements.length; i++) {
    let configPageElement = configPageElements[i];
    if (isExit) {
      configPageElement._resetOnExit();
    } else {
      if (appDataComponent.deviceConnectionState.deviceId == configPageElement.deviceId) {
        configPageElement._resetOnEntry();
      }
    }
  }
}

var resetSideMirrorPage = function (isExit, appDataComponent) {
  var mainPageElement = document.querySelector('#ti_custom_sidemirror_control_main');
  if (mainPageElement) {
    mainPageElement.index = -1;
  }
  var configPageElements = document.querySelectorAll('#ti_custom_sidemirror_control_config');
  for (let i = 0; i < configPageElements.length; i++) {
    let configPageElement = configPageElements[i];
    if (isExit) {
      configPageElement._resetOnExit();
    } else {
      if (appDataComponent.deviceConnectionState.deviceId == configPageElement.deviceId) {
        configPageElement._resetOnEntry();
      }
    }
  }
}

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function () {

  function readRegister(regDetail) {
    return gc.target.access.writeValue('gDRV89xx_Obj', {
      ManReadAddr: regDetail.addr,
      ManReadCmd: 1
    }).then(function (value) {
      return gc.target.access.readValue('gDeviceSpiReg.' + regDetail.objName)
    });
  }

  function bitExtracted(number, k, p) {
    return (((1 << k) - 1) & (number >> p));
  }

  function readICIDRegister() {
    var deferred = Q.defer();

    let registerData = {
      addr: 7,
      objName: 'config_ctrl_reg'
    };

    readRegister(registerData).then(function (regVal) {
      deferred.resolve(bitExtracted(regVal, 3, 4));
    }).fail(function (reason) {
      deferred.reject(reason);
    });

    return deferred.promise;
  }

  function getICID(deviceObj) {
    var deferred = Q.defer();

    if (deviceObj.nSleep === 1) {
      readICIDRegister().then(function (icidValue) {
        deferred.resolve(icidValue);
      }).fail(function (reason) {
        deferred.reject(reason);
      })
    } else {
      gc.target.access.writeValue('gDRV89xx_Obj', {
        nSleep: 1
      }).then(function () {
        return readICIDRegister().then(function (icidValue) {
          gc.target.access.writeValue('gDRV89xx_Obj', {
            nSleep: 0
          });
          deferred.resolve(icidValue);
        })
      }).fail(function (reason) {
        deferred.reject(reason);
      });
    }

    return deferred.promise;
  }

  function getRevId() {
    var deferred = Q.defer();

    gc.target.access.readValue('gDRV89xx_Obj.revID').then(function (val) {
      revIdText = val ? '1p1' : '1p0';
      deferred.resolve(val);
    }).catch(function (reason) {
      revIdText = `does not exist`;
      deferred.resolve(0);
    });

    return deferred.promise;
  }

  var getDeviceObj = function () {
    var deferred = Q.defer();

    setTimeout(function () {
      gc.target.access.readValue('gDRV89xx_Obj').then(function (value) {
        if (value.Firmware_Version != undefined) {
          firmwareVersion = Math.floor((value.Firmware_Version * 100)) / 100;
        } else {
          firmwareVersion = 'unknown';
        }
        deferred.resolve(value);
      }).fail(function (reason) {
        firmwareVersion = 'unknown';
        deferred.reject(reason);
      });
    }, 250);

    return deferred.promise;
  }

  var extendAboutBox = function () {
    var aboutBox = document.querySelector('ti-widget-aboutbox');
    if (aboutBox) {
      aboutBox.addEventListener('aboutbox_opening', function (event) {
        aboutBox.softwareManifestLink = "docs/manifest.html";
        aboutBox.licenseLink = "docs/license.pdf";
        aboutBox.appInfoTextHeading = 'Details';
        aboutBox.appInfoText = `Version of EVM’s Firmware: ${firmwareVersion}\nrevID: ${revIdText}`;
        aboutBox.numRowsInAppInfoTextArea = 2;
      });
    }
  };

  var handleInvalidRevId = function(revId) {
    let customPopup = document.querySelector('#ti_custom_popup');
    if(revId == 31) {
      customPopup._showPopup();
    } else {
      customPopup._closePopup();
    }
  }

  var regChangedListener = new gc.databind.IChangedListener();
  var updateDeviceConnectionState = function (oldState, newState) {
    if (newState) {
      getDeviceObj().then(function (deviceObj) {
        getICID(deviceObj).then(function (deviceIndex) {
          let supportedDevices = [8912, 8910, 8908, 8906, 8904];
          let deviceId = supportedDevices[deviceIndex] ? supportedDevices[deviceIndex] : 8912;
          getRevId().then(function (revId) {
            handleInvalidRevId(revId);
            updateDeviceConnectionStateInallComponents(true, deviceId, revId);
          })
        })
      }).fail(function (reason) {
        updateDeviceConnectionStateInallComponents(false);
      })
    } else {
      updateDeviceConnectionStateInallComponents(false);
    }
  };

  var updateDeviceConnectionStateInallComponents = function (state, deviceId, revId) {
    let deviceMapping = {
      '8904': 0,
      '8906': 1,
      '8908': 2,
      '8910': 3,
      '8912': 4,
    }

    let appDataComponent = document.querySelector('#ti_mdbu_custom_appdata');
    let customMultideviceAppManager = document.querySelector('#ti_widget_custom_multidevice_appmgr');

    if (state) {
      if(appDataComponent.deviceConnectionState.deviceId == 8912 || appDataComponent.deviceConnectionState.deviceId == 8910) {
        if(deviceId == 8904 || deviceId == 8906 || deviceId == 8908) {
          appResetFunction();
          return;
        }
      }
      if(appDataComponent.deviceConnectionState.deviceId == 8904 || appDataComponent.deviceConnectionState.deviceId == 8906 || appDataComponent.deviceConnectionState.deviceId == 8908) {
        if(deviceId == 8910 || deviceId == 8912) {
          appResetFunction();
          return;
        }
      }
      if ((appDataComponent.deviceConnectionState.deviceId && deviceMapping['' + deviceId] < deviceMapping['' + appDataComponent.deviceConnectionState.deviceId]) || (appDataComponent.deviceConnectionState.revId != undefined && appDataComponent.deviceConnectionState.revId != revId)) {
        appResetFunction();
        return;
      }

      customMultideviceAppManager._updateChipAccessibility(deviceMapping['' + deviceId]);
      customMultideviceAppManager._updateRevId(revId);
      templateObj.$.ti_widget_eventlog_view.log("info", "DRV" + deviceId + "-Q1 EVM connected");
      appDataComponent._setDeviceConnectionState({
        isConnecting: false,
        isConnected: true,
        deviceId: appDataComponent.deviceConnectionState.deviceId ? appDataComponent.deviceConnectionState.deviceId : deviceId,
        revId: revId
      });

    } else {
      customMultideviceAppManager._updateChipAccessibility(-1);
      customMultideviceAppManager._updateRevId(revId);
      revIdText = `does not exist`;
      firmwareVersion = 'unknown';
      gc.connectionManager.disconnect();
      templateObj.$.ti_widget_eventlog_view.log("info", "EVM disconnected");

    }
  }

  regChangedListener.onValueChanged = updateDeviceConnectionState.bind(this);
  gc.databind.registry.getBinding('my_model.$target_connected').addChangedListener(regChangedListener);

  // gc.databind.registry.bind('widget.ti_tile_settings.busVoltageValue2', 'widget.ti_tile_settings.fluxValue2', null, undefined);

  // gc.databind.registry.bind('widget.ti_tile_settings.systemEnableValue', 'gMotorVars.Flag_enableSys', null, undefined);
  // gc.databind.registry.bind('widget.ti_tile_settings.busVoltageValue2', 'gMotorVars.Flux_VpHz', null, undefined);
  // gc.databind.registry.bind('widget.ti_tile_settings.fluxValue2', 'gMotorVars.VdcBus_kV', null, undefined);

  /*
   *   Add custom computed value databindings here, using the following method:
   *
   *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
   *
   *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
   *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
   *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
   *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
   */

  // For example, a simple computed values based on simple expression
  // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

  // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
  // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

  // Event 1 to n bindings
  /*
  gc.databind.registry.bind('widget.date.value',
      // dependant bindings needed in order to compute the date, in name/value pairs.
      {
          weekday: 'widget.dayOfWeek.selectedText',
          day: 'widget.dayOfMonth.value',
          month: 'widget.month.selectedText',
          year: 'widget.year.value'
      },
      // getter for date computation
      function(values)
      {
          // compute and return the string value to bind to the widget with id 'date'
          return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
      }
  );
  */

  gc.autoDetectPortsFilter = function (transport, ports, info) {
    return ports.filter(function (port) {
      return port.manufacturer === "FTDI";
    });
  }
  extendAboutBox();
});

function statusChangeHdlr() {
  if( !gc || !gc.connectionManager) {
    return; 
  }
  if (catchedState == gc.connectionManager.status) {
    return;
  }
  catchedState = gc.connectionManager.status;
  console.log('Status', gc.connectionManager.status);
  let appDataComponent = document.querySelector('#ti_mdbu_custom_appdata');
  if (gc.connectionManager.status === "connecting") {
    appDataComponent._setDeviceConnectionState({
      isConnecting: true,
      isConnected: false
    });
  }
  if (gc.connectionManager.status === "disconnecting") {
    appDataComponent._setDeviceConnectionState({
      isConnecting: false,
      isConnected: false
    });
  }
}

document.addEventListener('WebComponentsReady', () => {
gc.connectionManager.addEventListener('status-changed', statusChangeHdlr);
});
statusChangeHdlr();

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function () {
  /*
   *   Add custom actions for menu items using the following api:
   *
   *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
   *
   *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
   *   param runable - function that performs the custom action.
   *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
   *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
   */

  // For example,
  // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

  // Alternatively, to programmatically disable a menu action at any time use:
  // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

  gc.nav.registerAction('show_introduction', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
      templateObj.$.ti_widget_menuitem.hideFullMenu();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });
  gc.nav.registerAction('show_registers', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
      templateObj.$.ti_widget_menuitem.hideFullMenu();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });
  gc.nav.registerAction('show_motor_control', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
      templateObj.$.ti_widget_menuitem.hideFullMenu();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('load_registers', function () {
    if ((templateObj) && (templateObj.$)) {
      gc.File.browseAndLoad(null, {
        bin: false
      }, function (contents, properties) {
        var regData = null;
        try {
          regData = JSON.parse(contents);
          if (regData) {
            templateObj.setRegisterData(regData);
            templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
          }
        } catch (ex) {
          templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
        }

      });
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('load_gui_register', function () {
    if ((templateObj) && (templateObj.$)) {
      var filePath = getGUIRegFilePath();
      var callback = function (contents) {
        try {
          if (contents) {
            templateObj.setRegisterData(contents);
            //templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
          }
        } catch (ex) {
          templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
        }
      }

      gc.fileCache.readJsonFile(filePath).then(callback);
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  var getGUIRegFilePath = function () {
    let customMultideviceAppManager = document.querySelector('#ti_widget_custom_multidevice_appmgr');
    switch (customMultideviceAppManager.selectedBoard) {
      case 'drv8904v0':
        return 'app/config/drv8904v0regs.json';
      case 'drv8906v0':
        return 'app/config/drv8906v0regs.json';
      case 'drv8908v0':
        return 'app/config/drv8908v0regs.json';
      case 'drv8910v0':
        return 'app/config/drv8910v0regs.json';
      case 'drv8912v0':
        return 'app/config/drv8912v0regs.json';
      case 'drv8904v1':
        return 'app/config/drv8904v1regs.json';
      case 'drv8906v1':
        return 'app/config/drv8906v1regs.json';
      case 'drv8908v1':
        return 'app/config/drv8908v1regs.json';
      case 'drv8910v1':
        return 'app/config/drv8910v1regs.json';
      case 'drv8912v1':
        return 'app/config/drv8912v1regs.json';
    }
  }

  var saveText = function (fname, text, mime) {
    var blob = new Blob([text], {
      type: mime
    });
    saveAs(blob, fname);
    return false;
  }
  gc.nav.registerAction('save_registers', function () {
    if ((templateObj) && (templateObj.$)) {
      var regData = templateObj.getRegisterData();
      var regDataJson = JSON.stringify(regData);
      if (gc.desktop.isDesktop()) {
        gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function (result, errorInfo) {
          if (errorInfo) {
            templateObj.logMsgAndNotify("error", "Save Registers: " + errorInfo, false, true);
          } else if ((result) && (result.localPath)) {
            templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: saved to " + result.localPath);
          }

        });
      } else {
        saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
      }
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('save_registers_as', function () {
    if ((templateObj) && (templateObj.$)) {
      var regData = templateObj.getRegisterData();
      var regDataJson = JSON.stringify(regData);
      if (gc.desktop.isDesktop()) {
        gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function (result, errorInfo) {
          if (errorInfo) {
            templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
          } else if ((result) && (result.localPath)) {
            templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: saved to " + result.localPath);
          }
        });
      } else {
        saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
      }
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('open_log_pane', function () {
    if ((templateObj) && (templateObj.$)) {
      templateObj.$.ti_widget_eventlog_view.openView();
    }
  }, function () {
    return true;
  }, function () {
    return true;
  });

  gc.nav.registerAction('open_scripting_window', function () {
    window.open('app/scripting.html', '_evm_scripting');
  }, function () {
    return true;
  }, function () {
    return true;
  });
});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;
var originalProductName = "";

// Wait for DOMContentLoaded event before trying to access the application template
var init = function () {
  templateObj = document.querySelector('#template_obj');

  // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
  // before trying to access components in the application.
  templateObj.addEventListener('dom-change', function () {
    if (initComplete) return;
    this.async(function () {
      initComplete = true;
      console.log("Application template has been stamped.");
      templateObj.$.ti_widget_toast.hideToast();
      templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
      registerModel = Polymer.dom(document).querySelector('ti-model-register');
      if (registerModel) {
        registerModel.addEventListener("usbHidModelReceivingData", function (event) {
          templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
        });
      } else {
        console.log("no ti-model-register component found in this application!")
      }

      // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
      // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

      // open the eventlog viewer when the user clicks on the book icon in the status bar:
      templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
      templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function (event) {
        templateObj.$.ti_widget_eventlog_view.toggleView();
      });

      // Expand vtabcontainer nav bar when user clicks on menu icon or 'Menu' label
      templateObj.toggleMenu = function (event) {
        templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
      };
      templateObj.$.ti_widget_icon_button_menu.addEventListener('click', templateObj.toggleMenu);
      templateObj.$.ti_widget_label_menu.addEventListener('click', templateObj.toggleMenu);

      // called when the user clicks the Write All Registers button:
      templateObj.writeRegs = function (event) {
        gc.target.access.writeValue('gDRV89xx_Obj', {WriteCmd: 1});
      };

      // called when the user clicks the Write Register button:
      templateObj.writeReg = function (event) {
        var detailsObj = event.detail;
        templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);

      };

      // called when the user clicks the Read Register button:
      templateObj.readReg = function (event) {
        var detailsObj = event.detail;
        templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
      }

      // called when the user clicks the Read All Registers button:
      templateObj.readRegs = function (event) {
        var detailsObj = event.detail;
        templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
      }

      // called when the user changes the immediate / deferred mode
      templateObj.updateModeChanged = function (event) {
        var detailsObj = event.detail;
        templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
      }

      // called when the user changes the auto read refresh rate
      templateObj.autoReadConfigChanged = function (event) {
        var detailsObj = event.detail;
        templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
      };


      if (templateObj.$.ti_widget_register_page) {
        // demo of handling when the user clicks the Write Register button
        templateObj.$.ti_widget_register_page.addEventListener("write_regs", templateObj.writeRegs);
        templateObj.$.ti_widget_register_page.addEventListener("write_reg", templateObj.writeReg);
        templateObj.$.ti_widget_register_page.addEventListener("read_reg", templateObj.readReg);
        templateObj.$.ti_widget_register_page.addEventListener("read_regs", templateObj.readRegs);
        templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", templateObj.updateModeChanged);
        templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", templateObj.autoReadConfigChanged);
      }

      var usbHidService = gc.services['ti-service-usbhid'];
      if (usbHidService) {
        usbHidService.addEventListener('usbHidError', function (event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
        })
      }

      templateObj.getRegisterData = function () {
        var regData = {
          signature: "oneui-register-data",
          data: []
        }
        var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
        for (var i = 0; i < regBlocks.length; i++) {
          var registersData = [];
          for (var r = 0; r < regBlocks[i].registers.length; r++) {
            var reg = regBlocks[i].registers[r];
            var regName = reg.name.toLowerCase();
            regName = regName.replace(/\&/g, "_");
            var item = {
              idx: r,
              id: regName,
              value: reg.value
            }
            registersData.push(item);
          }
          regData.data.push(registersData);
        }
        var strJson = JSON.stringify(regData);
        return strJson;
      };

      templateObj.setRegisterData = function (strJson) {
        var regData = JSON.parse(strJson);
        var ok = true;
        var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
        for (var b = 0; b < regBlocks.length; b++) {
          var registersData = [];
          for (var r = 0; r < regBlocks[b].registers.length; r++) {
            var reg = regBlocks[b].registers[r];
            var regName = reg.name.toLowerCase();
            regName = regName.replace(/\&/g, "_");
            item = regData.data[b][r];
            if (item.id === regName) {
              reg.value = item.value;
              var intValue = gc.utils.getValueFromHexString("" + reg.value);
              templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
            } else {
              var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
              templateObj.logMsgAndNotify("warning", msg, false, true);
            }
          }
        }
      };

      templateObj.showToastMsg = function (msgType, msg, durationInSeconds) {
        templateObj.$.ti_widget_toast.hideToast();
        templateObj.$.ti_widget_toast.message = msg;
        switch (msgType.toLowerCase()) {
          case "error":
            templateObj.$.ti_widget_toast.iconName = "error";
            templateObj.$.ti_widget_toast.backgroundColor = "red";
            break;
          case "warning":
            templateObj.$.ti_widget_toast.iconName = "warning";
            templateObj.$.ti_widget_toast.backgroundColor = "orange";
            break;
          default:
            templateObj.$.ti_widget_toast.iconName = "check";
            templateObj.$.ti_widget_toast.backgroundColor = "teal";
            break;
        }
        templateObj.$.ti_widget_toast.duration = +durationInSeconds;
        templateObj.$.ti_widget_toast.showToast();
        console.log("toast msg [" + msgType + "]: " + msg);
      };

      templateObj.logMsgAndNotify = function (msgType, msg, showInStatusBar, showInToast) {
        var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
        if (showInStatusBar) {
          var statusBar = gc.services['ti-widget-statusbar'];
          if (statusBar) {
            statusBar.statusString3 = msg;
            statusBar.setIcon(iconName);
          }
        }
        if (showInToast) {
          templateObj.showToastMsg(msgType, msg, 4);
        }
        templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
      };
      /* Demo of how to select a board by clicking on a custom button (e.g. with ti_widget_multidevice_app_mgr hidden):

      templateObj.selectBoard = function(event){
          var targetId = event.currentTarget.id;
          var button = document.querySelector('#'+targetId);
          templateObj.$.ti_widget_menubar.productName = button.label;
          var lcBoardName = button.label.toLowerCase().trim();
          templateObj.$.ti_widget_multidevice_app_mgr.selectedBoard = lcBoardName;
      };
      templateObj.$.ti_widget_button_board1.addEventListener('click',templateObj.selectBoard);
      templateObj.$.ti_widget_button_board2.addEventListener('click',templateObj.selectBoard);
      */

      /* Demo of how to open user guide documents in response to the user pressing an icon button:

      templateObj.openLinkOnClick = function(urlToOpen, event){
          window.open(urlToOpen);
      };
      templateObj.$.ti_widget_icon_button_userguide1.addEventListener('click',templateObj.openLinkOnClick.bind(templateObj,'app/assets/collaterals/DRV8304x_EVM_GUI_User_Guide.pdf'));
      templateObj.$.ti_widget_icon_button_userguide2.addEventListener('click',templateObj.openLinkOnClick.bind(templateObj,'app/assets/collaterals/DRV8304x_EVM_Sensored_User_Guide.pdf'));
      templateObj.$.ti_widget_icon_button_userguide3.addEventListener('click',templateObj.openLinkOnClick.bind(templateObj,'app/assets/collaterals/DRV8304x_EVM_Sensorless_User_Guide.pdf'));
          */
      originalProductName = templateObj.$.ti_widget_menubar.productName;

      templateObj.$.ti_widget_vtabcontainer.addEventListener('app_reset', appResetFunction);
      var customDeviceManger = document.querySelector('#ti_widget_custom_multidevice_appmgr');
      if (customDeviceManger) {
        customDeviceManger.addEventListener('device_selected', function (event) {
          let appDataComponent = document.querySelector('#ti_mdbu_custom_appdata');

          if (gc.connectionManager) {
            gc.connectionManager.connect();
          }
          // get the active device name from the register page
          if (event.detail && event.detail.deviceNames) {
            var deviceName = event.detail.deviceNames;
            var deviceLabelMapping = {
              'drv8904v0': 'DRV8904-Q1EVM',
              'drv8906v0': 'DRV8906-Q1EVM',
              'drv8908v0': 'DRV8908-Q1EVM',
              'drv8910v0': 'DRV8910-Q1EVM',
              'drv8912v0': 'DRV8912-Q1EVM',
              'drv8904v1': 'DRV8904-Q1EVM',
              'drv8906v1': 'DRV8906-Q1EVM',
              'drv8908v1': 'DRV8908-Q1EVM',
              'drv8910v1': 'DRV8910-Q1EVM',
              'drv8912v1': 'DRV8912-Q1EVM'
            }
            if (deviceName.length > 0) {
              templateObj.$.ti_widget_menubar.productName = deviceLabelMapping[deviceName];
            }
          }
          resetHighLevelPage(false, appDataComponent);
          updateFileMenuOptionsVisibility(false);
        });

      };
      updateFileMenuOptionsVisibility(true);
    }, 1);
  });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
  init();
} else {
  document.addEventListener('DOMContentLoaded', init);
}